// Fig. 16.21: Validate.cs
// Validate user information using regular expressions.
using System;
using System.Text.RegularExpressions;
using System.Windows.Forms;

public partial class ValidateForm : Form
{
   // default constructor
   public ValidateForm()
   {
      InitializeComponent();
   } // end constructor

   // handles OkButton Click event
   private void okButton_Click( object sender, EventArgs e )
   {
      // ensures no TextBoxes are empty
      if ( lastNameTextBox.Text == "" || firstNameTextBox.Text == "" ||
         addressTextBox.Text == "" || cityTextBox.Text == "" ||
         stateTextBox.Text == "" || zipCodeTextBox.Text == "" ||
         phoneTextBox.Text == "" )
      {
         // display popup box
         MessageBox.Show( "Please fill in all fields", "Error",
            MessageBoxButtons.OK, MessageBoxIcon.Error );
         lastNameTextBox.Focus(); // set focus to lastNameTextBox
         return;
      } // end if

      // if last name format invalid show message
      if ( !Regex.Match( lastNameTextBox.Text,
         "^[A-Z][a-zA-Z]*$" ).Success )
      {
         // last name was incorrect
         MessageBox.Show( "Invalid last name", "Message",
            MessageBoxButtons.OK, MessageBoxIcon.Error );
         lastNameTextBox.Focus();
         return;
      } // end if

      // if first name format invalid show message
      if ( !Regex.Match( firstNameTextBox.Text,
         "^[A-Z][a-zA-Z]*$" ).Success )
      {
         // first name was incorrect
         MessageBox.Show( "Invalid first name", "Message",
            MessageBoxButtons.OK, MessageBoxIcon.Error );
         firstNameTextBox.Focus();
         return;
      } // end if

      // if address format invalid show message
      if ( !Regex.Match( addressTextBox.Text,
         @"^[0-9]+\s+([a-zA-Z]+|[a-zA-Z]+\s[a-zA-Z]+)$" ).Success )
      {
         // address was incorrect
         MessageBox.Show( "Invalid address", "Message",
            MessageBoxButtons.OK, MessageBoxIcon.Error );
         addressTextBox.Focus();
         return;
      } // end if

      // if city format invalid show message
      if ( !Regex.Match( cityTextBox.Text,
         @"^([a-zA-Z]+|[a-zA-Z]+\s[a-zA-Z]+)$" ).Success )
      {
         // city was incorrect
         MessageBox.Show( "Invalid city", "Message",
            MessageBoxButtons.OK, MessageBoxIcon.Error );
         cityTextBox.Focus();
         return;
      } // end if

      // if state format invalid show message
      if ( !Regex.Match( stateTextBox.Text,
         @"^([a-zA-Z]+|[a-zA-Z]+\s[a-zA-Z]+)$" ).Success )
      {
         // state was incorrect
         MessageBox.Show( "Invalid state", "Message",
            MessageBoxButtons.OK, MessageBoxIcon.Error );
         stateTextBox.Focus();
         return;
      } // end if

      // if zip code format invalid show message
      if ( !Regex.Match( zipCodeTextBox.Text, @"^\d{5}$" ).Success )
      {
         // zip was incorrect
         MessageBox.Show( "Invalid zip code", "Message",
            MessageBoxButtons.OK, MessageBoxIcon.Error );
         zipCodeTextBox.Focus();
         return;
      } // end if

      // if phone number format invalid show message
      if ( !Regex.Match( phoneTextBox.Text,
         @"^[1-9]\d{2}-[1-9]\d{2}-\d{4}$" ).Success )
      {
         // phone number was incorrect
         MessageBox.Show( "Invalid phone number", "Message",
            MessageBoxButtons.OK, MessageBoxIcon.Error );
         phoneTextBox.Focus();
         return;
      } // end if

      // information is valid, signal user and exit application
      this.Hide(); // hide main window while MessageBox displays
      MessageBox.Show( "Thank You!", "Information Correct",
         MessageBoxButtons.OK, MessageBoxIcon.Information );
      Application.Exit();
   } // end method okButton_Click
} // end class ValidateForm

/**************************************************************************
 * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
 * Pearson Education, Inc. All Rights Reserved.                           *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *************************************************************************/