// Fig. 22.9: HugeInteger.cs
// HugeInteger Web service performs operations on large integers.
using System;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;

[ WebService( Namespace = "http://www.deitel.com/",
   Description = "A Web service that provides methods for" +
   " manipulating large integer values" ) ]
[ WebServiceBinding( ConformsTo = WsiProfiles.BasicProfile1_1 ) ]
public class HugeInteger : System.Web.Services.WebService
{
   private const int MAXIMUM = 100; // maximum number of digits
   public int[] number; // array representing the huge integer

   // default constructor
   public HugeInteger()
   {
      number = new int[ MAXIMUM ];
   } // end default constructor

   // indexer that accepts an integer parameter
   public int this[ int index ] 
   {
      get
      {
         return number[ index ];
      } // end get

      set
      {
         number[ index ] = value;
      } // end set
   } // end indexer

   // returns string representation of HugeInteger
   public override string ToString()
   {
      string returnString = "";

      foreach ( int i in number )
         returnString = i + returnString;

      return returnString;
   } // end method ToString

   // creates HugeInteger based on argument
   public static HugeInteger FromString( string value ) 
   {
      // create temporary HugeInteger to be returned by the method
      HugeInteger parsedInteger = new HugeInteger();

      for ( int i = 0 ; i < value.Length; i++ )
         parsedInteger[ i ] = Int32.Parse( 
            value[ value.Length - i - 1 ].ToString() );

      return parsedInteger;
   } // end method FromString

   // WebMethod that adds integers represented by the String arguments
   [ WebMethod( Description="Adds two huge integers." ) ]
   public string Add( string first, string second )
   {
      int carry = 0;
      HugeInteger operand1 = HugeInteger.FromString( first );
      HugeInteger operand2 = HugeInteger.FromString( second );
      HugeInteger result = new HugeInteger(); // stores result of addition

      // perform addition algorithm for each digit
      for ( int i = 0; i < MAXIMUM; i++ )
      {
         // add two digits in same column,
         // result is their sum plus carry from 
         // previous operation, modulo 10
         result[ i ] = 
            ( operand1[ i ] + operand2[ i ] + carry ) % 10;

         // set carry to remainder of dividing sums of two digits by 10
         carry = ( operand1[ i ] + operand2[ i ] + carry ) / 10;
      } // end for

      return result.ToString();
   } // end method Add

   // WebMethod that subtracts integers 
   // represented by the string arguments
   [ WebMethod( Description="Subtracts two huge integers." ) ] 
   public string Subtract( string first, string second )
   {
      HugeInteger operand1 = HugeInteger.FromString( first );
      HugeInteger operand2 = HugeInteger.FromString( second );
      HugeInteger result = new HugeInteger();

      // subtract bottom digit from top digit 
      for ( int i = 0; i < MAXIMUM; i++ )
      {
         // if top digit is smaller than bottom digit we need to borrow
         if ( operand1[ i ] < operand2[ i ] )
            Borrow( operand1, i );
         
         // subtract bottom from top
         result[ i ] = operand1[ i ] - operand2[ i ];
      } // end for

      return result.ToString();
   } // end method Subtract

   // borrow 1 from next digit
   private void Borrow( HugeInteger hugeInteger, int place )
   {
      // if no place to borrow from, signal problem
      if ( place >= MAXIMUM - 1 ) 
         throw new ArgumentException();

      // otherwise if next digit is zero, borrow from column to left
      else if ( hugeInteger[ place + 1 ] == 0 )
         Borrow( hugeInteger, place + 1 );

      // add ten to current place because we borrowed and subtract  
      // one from previous digit--this is the digit we borrowed from
      hugeInteger[ place ] += 10;
      hugeInteger[ place + 1 ]--;
   } // end method Borrow

   // WebMethod that returns true if first integer is bigger than second
   [ WebMethod( Description="Determines whether the first integer is " +
      "larger than the second integer." ) ]
   public bool Bigger( string first, string second ) 
   {
      char[] zeros = { '0' };

      try
      {
         // if elimination of all zeros from result
         // of subtraction is an empty string,
         // numbers are equal, so return false, otherwise return true
         if ( Subtract( first, second ).Trim( zeros ) == "" )
            return false;
         else
            return true;
      } // end try
      // if ArgumentException occurs, first 
      // number was smaller, so return false
      catch ( ArgumentException exception )
      {
         return false;
      } // end catch
   } // end method Bigger

   // WebMethod returns true if first integer is smaller than second
   [ WebMethod( Description="Determines whether the first integer " +
      "is smaller than the second integer." ) ] 
   public bool Smaller( string first, string second ) 
   {
      // if second is bigger than first, then first is smaller than second
      return Bigger( second, first );
   } // end method Smaller

   // WebMethod that returns true if two integers are equal
   [ WebMethod( Description="Determines whether the first integer " +
      "is equal to the second integer." ) ] 
   public bool EqualTo( string first, string second )
   {
      // if either first is bigger than second, or first is 
      // smaller than second, they are not equal
      if ( Bigger( first, second ) || Smaller( first, second ) ) 
         return false;
      else
         return true;
   } // end method EqualTo
} // end class HugeInteger

/**************************************************************************
 * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
 * Pearson Education, Inc. All Rights Reserved.                           *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 **************************************************************************/