// Fig. 22.17: UsingHugeIntegerService.cs
// Using the HugeInteger Web Service.
using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using System.Web.Services.Protocols;

namespace UsingHugeIntegerWebService
{
   public partial class UsingHugeIntegerServiceForm : Form
   {
      public UsingHugeIntegerServiceForm()
      {
         InitializeComponent();
      } // end constructor

      // declare a reference to Web service
      private localhost.HugeInteger remoteInteger; 

      private char[] zeros = { '0' }; // character to trim from strings

      // instantiates object to interact with Web service
      private void UsingHugeIntegerServiceForm_Load( object sender, 
         EventArgs e )
      {
         // instantiate remoteInteger
         remoteInteger = new localhost.HugeInteger();
      } // end method UsingHugeIntegerServiceForm_Load

      // adds two numbers input by user
      private void addButton_Click( object sender, EventArgs e )
      {
         // make sure numbers do not exceed 100 digits and that both
         // are not 100 digits long, which would result in overflow
         if ( firstTextBox.Text.Length > 100 ||
            secondTextBox.Text.Length > 100 ||
            ( firstTextBox.Text.Length == 100 &&
            secondTextBox.Text.Length == 100) )
         {
            MessageBox.Show( "HugeIntegers must not be more " +
               "than 100 digits\r\nBoth integers cannot be " +
               "of length 100: this causes an overflow", "Error",
               MessageBoxButtons.OK, MessageBoxIcon.Information );
            return;
         } // end if

         // perform addition
         resultLabel.Text = remoteInteger.Add(
            firstTextBox.Text, secondTextBox.Text ).TrimStart( zeros );
      } // end method addButton_Click

      // subtracts two numbers input by user
      private void subtractButton_Click( object sender, EventArgs e )
      {
         // make sure HugeIntegers do not exceed 100 digits
         if ( SizeCheck( firstTextBox, secondTextBox ) )
            return;

         // perform subtraction
         try
         {
            string result = remoteInteger.Subtract( 
               firstTextBox.Text, secondTextBox.Text ).TrimStart( zeros );

            if ( result == "" ) 
               resultLabel.Text = "0";
            else
               resultLabel.Text = result;

         } // end try

         // if WebMethod throws an exception,
         // then first argument was smaller than second
         catch ( SoapException exception )
         {
            MessageBox.Show(
               "First argument was smaller than the second" );
         } // end catch
      } // end method subtractButton_Click

      // determines whether first number
      // input by user is larger than second
      private void largerButton_Click( object sender, EventArgs e )
      {
         // make sure HugeIntegers do not exceed 100 digits
         if ( SizeCheck( firstTextBox, secondTextBox ) )
            return;

         // call Web-service method to determine if
         // first integer is larger than the second
         if ( remoteInteger.Bigger( firstTextBox.Text, 
            secondTextBox.Text ) )
            resultLabel.Text = firstTextBox.Text.TrimStart( zeros ) + 
               " is larger than " + 
               secondTextBox.Text.TrimStart( zeros );
         else
            resultLabel.Text = firstTextBox.Text.TrimStart( zeros ) + 
               " is not larger than " + 
               secondTextBox.Text.TrimStart( zeros );
      } // end method largerButton_Click

      // determines whether first number
      // input by user is smaller than second
      private void smallerButton_Click( object sender, EventArgs e )
      {
         // make sure HugeIntegers do not exceed 100 digits
         if ( SizeCheck( firstTextBox, secondTextBox ) )
            return;

         // call Web-service method to determine if
         // first integer is smaller than second
         if ( remoteInteger.Smaller( firstTextBox.Text, 
            secondTextBox.Text ) )
            resultLabel.Text = firstTextBox.Text.TrimStart( zeros ) + 
               " is smaller than " + 
               secondTextBox.Text.TrimStart( zeros );
         else
            resultLabel.Text = firstTextBox.Text.TrimStart( zeros ) + 
               " is not smaller than " +
               secondTextBox.Text.TrimStart( zeros );
      } // end method smallerButton_Click

      // determines whether two numbers input by user are equal
      private void equalButton_Click( object sender, EventArgs e )
      {
         // make sure HugeIntegers do not exceed 100 digits
         if ( SizeCheck( firstTextBox, secondTextBox ) )
            return;

         // call Web-service method to determine if integers are equal
         if ( remoteInteger.EqualTo( firstTextBox.Text, 
            secondTextBox.Text ) )
            resultLabel.Text = firstTextBox.Text.TrimStart( zeros ) + 
               " is equal to " + secondTextBox.Text.TrimStart( zeros );
         else
            resultLabel.Text = firstTextBox.Text.TrimStart( zeros ) + 
               " is not equal to " + 
               secondTextBox.Text.TrimStart( zeros );
      } // end method equalButton_Click

      // determines whether numbers input by user are too big
      private bool SizeCheck( TextBox first, TextBox second )
      {
         // display an error message if either number has too many digits
         if ( ( first.Text.Length > 100 ) ||
            ( second.Text.Length > 100 ) )
         {
            MessageBox.Show( "HugeIntegers must be less than 100 digits" ,
               "Error", MessageBoxButtons.OK, MessageBoxIcon.Information);
            return true;
         } // end if

         return false;
      } // end method SizeCheck
   } // end class UsingHugeIntegerServiceForm
} // end namespace UsingHugeIntegerWebService

/**************************************************************************
 * (C) Copyright 1992-2006 by Deitel + Associates, Inc. and               *
 * Pearson Education, Inc. All Rights Reserved.                           *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 **************************************************************************/
