// Fig. 22.18: BlackjackService.cs
// Blackjack Web Service deals and counts cards.
using System;
using System.Web;
using System.Web.Services;
using System.Web.Services.Protocols;
using System.Collections;

[ WebService( Namespace = "http://www.deitel.com/", Description= 
   "A Web service that deals and counts cards for the game Blackjack" ) ]
[ WebServiceBinding( ConformsTo = WsiProfiles.BasicProfile1_1 ) ]
public class BlackjackService : System.Web.Services.WebService
{
   // deals card that has not yet been dealt
   [ WebMethod( EnableSession=true, 
      Description="Deal a new card from the deck." ) ] 
   public string DealCard()
   {
      string card = "2 2";

      // get client's deck
      ArrayList deck = ( ArrayList )( Session[ "deck" ] );
      card = Convert.ToString( deck[ 0 ] );
      deck.RemoveAt( 0 );
      return card;
   } // end method DealCard

   // creates and shuffles a deck of cards
   [ WebMethod( EnableSession=true,
      Description="Create and shuffle a deck of cards." ) ]
   public void Shuffle()
   {
      object temporary; // holds card temporarily during swapping
      Random randomObject = new Random(); // generates random numbers
      int newIndex; // index of randomly selected card
      ArrayList deck = new ArrayList(); // stores deck of cards (strings)

      // generate all possible cards
      for ( int i = 1; i <= 13; i++ ) // loop through face values
         for ( int j = 0; j <= 3; j++ ) // loop through suits
            deck.Add( i + " " + j ); // add card (string) to deck

      // shuffles deck by swapping each card with another card randomly
      for ( int i = 0; i < deck.Count; i++ )
      {
         // get random index
         newIndex = randomObject.Next( deck.Count - 1 );
         temporary = deck[ i ]; // save current card in temporary variable
         deck[ i ] = deck[ newIndex ]; // copy randomly selected card
         deck[ newIndex ] = temporary; // copy current card back into deck
      } // end for

      // add this deck to user's session state
      Session.Add( "deck", deck );
   } // end method Shuffle

   // computes value of hand
   [ WebMethod( Description= 
      "Compute a numerical value for the current hand." ) ] 
   public int GetHandValue( string dealt ) 
   {
      // split string containing all cards
      char[] tab = { '\t' };
      string[] cards = dealt.Split( tab ); // get array of cards
      int total = 0; // total value of cards in hand
      int face; // face of the current card
      int aceCount = 0; // number of aces in hand

      // loop through the cards in the hand
      foreach ( string drawn in cards )
      {
         // get face of card
         face = Int32.Parse( drawn.Substring( 0, drawn.IndexOf( " " ) ) );

         switch ( face )
         {
            case 1: // if ace, increment aceCount
               aceCount++;
               break;
            case 11: // if jack add 10
            case 12: // if queen add 10
            case 13: // if king add 10
               total += 10;
               break;
            default: // otherwise, add value of face 
               total += face;
               break;
         } // end switch
      } // end foreach

      // if there are any aces, calculate optimum total
      if ( aceCount > 0 )
      {
         // if it is possible to count one ace as 11, and the rest
         // as 1 each, do so; otherwise, count all aces as 1 each
         if ( total + 11 + aceCount - 1 <= 21 ) 
            total += 11 + aceCount - 1;
         else
            total += aceCount;
      } // end if

      return total;
   } // end method GetHandValue
} // end class BlackjackService

/**************************************************************************
 * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
 * Pearson Education, Inc. All Rights Reserved.                           *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 **************************************************************************/
