// Fig. 23.3: PacketServer.cs
// Set up a server that will receive packets from a
// client and send the packets back to the client.
using System;
using System.Windows.Forms;
using System.Net;        
using System.Net.Sockets;
using System.Threading;  

public partial class PacketServerForm : Form
{
   public PacketServerForm()
   {
      InitializeComponent();
   } // end constructor

   private UdpClient client;       
   private IPEndPoint receivePoint;

   // initialize variables and thread for receiving packets
   private void PacketServerForm_Load( object sender, EventArgs e )
   {
      client = new UdpClient( 50000 );                        
      receivePoint = new IPEndPoint( new IPAddress( 0 ), 0 );
      Thread readThread =                                    
         new Thread( new ThreadStart( WaitForPackets ) );    
      readThread.Start();                                    
   } // end method PacketServerForm_Load

   // shut down the server
   private void PacketServerForm_FormClosing( object sender, 
      FormClosingEventArgs e )
   {         
      System.Environment.Exit( System.Environment.ExitCode );
   } // end method PacketServerForm_FormClosing

   // delegate that allows method DisplayMessage to be called
   // in the thread that creates and maintains the GUI       
   private delegate void DisplayDelegate( string message );  

   // method DisplayMessage sets displayTextBox's Text property
   // in a thread-safe manner
   private void DisplayMessage( string message )
   {
      // if modifying displayTextBox is not thread safe
      if ( displayTextBox.InvokeRequired )
      {
         // use inherited method Invoke to execute DisplayMessage
         // via a delegate                                       
         Invoke( new DisplayDelegate( DisplayMessage ),          
            new object[] { message } );                          
      } // end if
      else // OK to modify displayTextBox in current thread
         displayTextBox.Text += message;
   } // end method DisplayMessage

   // wait for a packet to arrive
   public void WaitForPackets()
   {
      while ( true )
      {
         // set up packet
         byte[] data = client.Receive( ref receivePoint );
         DisplayMessage( "\r\nPacket received:" +
            "\r\nLength: " + data.Length +
            "\r\nContaining: " + 
            System.Text.Encoding.ASCII.GetString( data ) );           

         // echo information from packet back to client
         DisplayMessage( "\r\n\r\nEcho data back to client..." );
         client.Send( data, data.Length, receivePoint );
         DisplayMessage( "\r\nPacket sent\r\n" );
      } // end while
   } // end method WaitForPackets
} // end class PacketServerForm

/**************************************************************************
 * (C) Copyright 1992-2006 by Deitel & Associates, Inc. and               *
 * Pearson Education, Inc. All Rights Reserved.                           *
 *                                                                        *
 * DISCLAIMER: The authors and publisher of this book have used their     *
 * best efforts in preparing the book. These efforts include the          *
 * development, research, and testing of the theories and programs        *
 * to determine their effectiveness. The authors and publisher make       *
 * no warranty of any kind, expressed or implied, with regard to these    *
 * programs or to the documentation contained in these books. The authors *
 * and publisher shall not be liable in any event for incidental or       *
 * consequential damages in connection with, or arising out of, the       *
 * furnishing, performance, or use of these programs.                     *
 *************************************************************************/